/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Execute arbitrary authenticate commands
 *
 * Copyright (C) 2005, IonIdea
 *
 * Michael Vyazovsky <michael.vyazovsky@ionidea.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>

#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/app.h>
#include <asterisk/astdb.h>
#include <asterisk/utils.h>
#include <asterisk/options.h>

static char *tdesc = "Authentication Application";

static char *app_name = "Authenticate_LDAP";
static char *app_ldap_name = "LDAPget";

static char *synopsis = "Authenticate a user";

static char *descrip =
"  Authenticate_LDAP(VARNAME=config-file-section/\%s): Retrieves a value from the LDAP\n"
"directory and stores it in the given variable and asteriskdb.userfield.\n\n"
"Returns 0 if the user enters a valid password within three\n"
"tries, or -1 on hangup.  If the priority n+101 exists and invalid\n"
"authentication was entered, processing will jump to n+101 and 0 will be returned.\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int auth_ldap_exec(struct ast_channel *chan, void *data) {

	int res	= 0;
	int len_data = 0;
	int len_query = 0;
	int retries;
	char *prompt;
	char *ldap_value;
	char *varname;	
	char *query, *frmt;
	char accesscode[16] = "";
	struct localuser *u;
	struct ast_app *app_ldap;

	app_ldap = pbx_findapp(app_ldap_name);
	if (!app_ldap) {
		ast_log(LOG_WARNING, "Could not find application (%s)\n", app_ldap_name);
		return -1;
	}

	if (!data || ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "Authenticate LDAP: requires an argument\n");
		return -1;
	}

	if(!strchr (data, '=')) {
        	ast_log (LOG_DEBUG, "Authenticate LDAP: ignoring, no parameters\n");
	        return -1;
	}

	len_data	= strlen(data) + 1;
	len_query	= len_data + 16;
	frmt 		= malloc(len_data);
	query 		= malloc(len_query);
	varname 	= malloc(len_data);
        if(!query || !frmt || !varname) {
		ast_log (LOG_DEBUG, "Authenticate LDAP: memory allocation failed\n");
		return -1;
	}
	
	memset(frmt, 0, len_data);
	memcpy(frmt, data, len_data - 1);
	memset(varname, 0, len_data);
	varname = strsep(&data, "=");
        if (!varname) {
		ast_log (LOG_DEBUG, "Authenticate LDAP: ignoring, syntax error in argument\n");
		free(varname);
		free(query);
		free(frmt);
    		return -1;
	}

    if(option_verbose > 2)
        ast_verbose (VERBOSE_PREFIX_3 "Authenticate LDAP: argument: (%s) varname (%s)\n", frmt, varname);

	LOCAL_USER_ADD(u);
	
	if (chan->_state != AST_STATE_UP) {
		res = ast_answer(chan);
		if (res) {
			LOCAL_USER_REMOVE(u);
			free(varname);
			free(query);
			free(frmt);
			return -1;
		}
	}

	/* Start asking for accesscode */
	prompt = "agent-pass";
	
	for (retries = 0; retries < 3; retries++) {
	
		memset(accesscode, 0, sizeof(accesscode));
		res = ast_app_getdata(chan, prompt, accesscode, sizeof(accesscode) - 2, 0);
		if (res < 0) break;
		res = 0;
		memset(query, 0, len_query);
		snprintf(query, len_query - 1, frmt, accesscode);

    		if (option_verbose > 2) {
        	    ast_verbose(VERBOSE_PREFIX_3 "Authenticate LDAP: Executing Application: (%s) Query: (%s)\n", app_ldap_name, query);
		}

		res = pbx_exec(chan, app_ldap, query, 1);
		if (res < 0) break;
		ldap_value = pbx_builtin_getvar_helper(chan, varname);
		if (ldap_value) break;

		prompt="auth-incorrect";
	}

	if ((retries < 3) && !res) {
		if (strlen(ldap_value))
			ast_cdr_setuserfield(chan, ldap_value);

		res = ast_streamfile(chan, "auth-thankyou", chan->language);
		if (!res)
			res = ast_waitstream(chan, "");
	} else {
		if (ast_exists_extension(chan, chan->context, chan->exten, chan->priority + 101, chan->cid.cid_num)) {
			chan->priority+=100;
			res = 0;
		} else {
			if (!ast_streamfile(chan, "vm-goodbye", chan->language))
				res = ast_waitstream(chan, "");
			res = -1;
		}
	}
	LOCAL_USER_REMOVE(u);
	free(varname);
	free(query);
	free(frmt);

	return res;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app_name);
}

int load_module(void)
{
	return ast_register_application(app_name, auth_ldap_exec, synopsis, descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
